using UnityEngine;
using UnityEngine.UI;
using System;
using System.Collections.Generic;
using System.Collections;

public class Batty : MonoBehaviour
{
  public Camera followCamera;

  public Vector2 forceDirection;

  public Vector2 direction = new Vector2(1.0f, 1.0f);

  public Rigidbody2D rb;

  public Text gameOverScoreLabel;

  float health;
  public float maxHealth;
  public float damageAmount;

  public RectTransform healthBar;

  public float healthBarWidth;

  public GameObject gameOverView;

  public SpriteRenderer batSprite;

  public FXUnit hitFXPrefab;

  // float cameraOffset;

  bool initialized = false;
  bool hasSung = false;

  bool acceptInput = true;

  public EchoWave echoPrefab;

  public BellyItcher bellyItcher;

  public float fireRate = 0.5f;

  float cameraOffset = -2.0f;

  IEnumerator pitchEnumerator;

  void Start()
  {
    gameOverView.SetActive(false);
    // cameraOffset = transform.position.x; 
    health = maxHealth;
    Vector3 pos = followCamera.transform.position;
    // pos.x -= cameraOffset;
    pos.z = 0;
    transform.position = pos;
    healthBarWidth = healthBar.sizeDelta.x;

    pitchEnumerator = pitchRoutine();
    StartCoroutine(pitchEnumerator);
  }

  void gameOver()
  {
    rb.simulated = false;
    acceptInput = false;
    gameOverView.SetActive(true);
    StopCoroutine(pitchEnumerator);

    float inches = transform.position.x - cameraOffset;
    if(health > 0)
    {
      gameOverScoreLabel.text = "You made it back to the nest!";
    }else{
      gameOverScoreLabel.text = "You flew "+inches + " inches.";
    }
  }

  void Update()
  {
    Vector3 pos = followCamera.transform.position;
    if(direction.x > 0)
    pos.x = transform.position.x - cameraOffset;
    pos.y = transform.position.y;
    Vector3 diff = pos - followCamera.transform.position;
    followCamera.transform.position += diff * 0.2f;

    if(acceptInput)
    {
      if(Input.GetKeyDown(KeyCode.Z))
      {
        direction = new Vector2(-direction.x, direction.y);
        batSprite.flipX = !batSprite.flipX;
      }
      if(Input.GetKeyDown(KeyCode.Space))
      {
        if(!initialized && hasSung)
        {
          initialized = true;
          rb.simulated = true;
        }
        rb.AddForce(new Vector2(forceDirection.x * direction.x, forceDirection.y * direction.y), ForceMode2D.Impulse);
      }
    }

    //update health bar.
    float healthPercent = health / maxHealth;
    Vector2 sd = healthBar.sizeDelta;

    float xDiff = healthBarWidth * healthPercent - sd.x;
    sd.x +=(xDiff * 0.8f);
    healthBar.sizeDelta = sd;

  }


  void OnCollisionEnter2D(Collision2D coll) {
    Fruit f = coll.transform.GetComponent<Fruit>();
    if(f != null)
    {
      health += f.health;
      if(health > maxHealth)
      {
        health = maxHealth;
      }
      f.Eat();

      Wiiiiiiiinn w = f.GetComponent<Wiiiiiiiinn>();
      if(w != null)
      {
        Win();
      }
    }else{
      health -= damageAmount;
      FXUnit hitFX = (FXUnit)Instantiate(hitFXPrefab);
      hitFX.transform.position = transform.position;
    }

    if(health <= 0)
    {
      gameOver();
    }
  }

  IEnumerator pitchRoutine()
  {
    float highestPitch = PlayerPrefs.GetFloat("highestPitch", 0.0f);
    float lowestPitch = PlayerPrefs.GetFloat("lowestPitch", 0.0f);
    float diffPitch = highestPitch - lowestPitch;
    float upperAngleMax = -90.0f;
    float lowerAngleMax = 90.0f;
    float diffAngle = lowerAngleMax - upperAngleMax;


    while(true)
    {
      float pitch = bellyItcher.pitch;

      pitch = Math.Min(pitch, highestPitch);

      if(pitch > lowestPitch && pitch <= highestPitch)
      {
        hasSung = true;
        float percent = (pitch - lowestPitch) / diffPitch;
        float angle = (diffAngle * percent) + upperAngleMax;
        if(direction.x < 0)
        {

          angle = 180.0f - angle;
        }
        EchoWave wave = (EchoWave)Instantiate(echoPrefab);
        wave.transform.eulerAngles = new Vector3(0.0f, 0.0f, angle);
        wave.transform.position = transform.position;
        yield return new WaitForSeconds(fireRate);
      }
      yield return null;
    }
  }


  public void Win()
  {
    gameOver();
  }


}
