using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;
using System;
using System.Collections.Generic;
using System.Collections;

public class CalibrationTest : MonoBehaviour
{
  public Text instructionLabel;
  public Text currentPitchLabel;
  public Text lowestPitchLabel;
  public Text highestPitchLabel;
  public Text buttonLabel;

  public Text prefsInfoLabel;

  public BellyItcher bellyItcher;

  [System.NonSerialized][HideInInspector]
  public float currentPitch;
  [System.NonSerialized][HideInInspector]
  public float lowestPitch = 1000.0f;
  [System.NonSerialized][HideInInspector]
  public float highestPitch = 0.0f;

  [System.NonSerialized][HideInInspector]
  public bool calibrating = false;

  void Start()
  {
    StartCoroutine(calibration());
  }

  IEnumerator calibration()
  {
    highestPitchLabel.text = "";
    float savedLowest = PlayerPrefs.GetFloat("lowestPitch", 0.0f);
    float savedHighest = PlayerPrefs.GetFloat("highestPitch", 0.0f);

    prefsInfoLabel.text = "CURRENT CALIBRATION: Lowest: "+Mathf.FloorToInt(savedLowest) + "hz Highest: " + Mathf.FloorToInt(savedHighest) + "hz" ;

    calibrating = true;
    instructionLabel.text = "Sing as <color=red>low</color> as you can go!";
    while(calibrating)
    {
      float pitch = bellyItcher.pitchTracker.CurrentPitchRecord.Pitch;
      if(pitch > 75.0f)
      {
        lowestPitch = lowestPitch < pitch ? lowestPitch : pitch;
      }
      currentPitchLabel.text = Mathf.FloorToInt(pitch) + "hz";
      if(lowestPitch > 500.0f)
      {
        lowestPitchLabel.text = "";
      }else{
        lowestPitchLabel.text = "Lowest:<color=red>\n"+Mathf.FloorToInt(lowestPitch) + "hz</color>";
      }
      yield return null;
    }

    instructionLabel.text = "Sing as <color=green>high</color> as you can flyyyy!";

    calibrating = true;

    while(calibrating)
    {
      float pitch = bellyItcher.pitchTracker.CurrentPitchRecord.Pitch;
      if(pitch > 100.0f)
      {
        highestPitch = highestPitch > pitch ? highestPitch : pitch;
      }
      currentPitchLabel.text = Mathf.FloorToInt(pitch) + "hz";
      if(highestPitch < 50.0f)
      {
        highestPitchLabel.text = "";
      }else{
        highestPitchLabel.text = "Highest:<color=green>\n"+Mathf.FloorToInt(highestPitch) + "hz</color>";
      }
      yield return null;
    }

    calibrating = true;
    currentPitchLabel.text = "";
    instructionLabel.text = "Save these settings?";
    buttonLabel.text = "Save";

    while(calibrating)
    {
      yield return null;
    }


    PlayerPrefs.SetFloat("lowestPitch", lowestPitch);
    PlayerPrefs.SetFloat("highestPitch", highestPitch);
    PlayerPrefs.Save();
    Debug.Log("[CalibrationTest] SAVED!");
    SceneManager.LoadScene("mainmenu");

  }

  public void next()
  {
    calibrating = false;
  }


}
