using UnityEngine;
using System;
using System.Collections.Generic;
using System.Collections;

public class FrequencyBars : MonoBehaviour
{
  public AudioSource aSource;
  public List<float> frequencies;

  public float peakSample = 0.0f;
  // Min = 64. Max = 8192.  
  public int samples = 256;

  float[] data;

  public List<FrequencyBand> bands;

  public MeterDisplay meterDisplayPrefab;
  MeterDisplay[] displays = new MeterDisplay[0];

  public bool isRefresh = false;

  void Start()
  {
    aSource = GetComponent<AudioSource>();
    refresh();

  }

  void refresh()
  {
    data = new float[samples];
    int index = 0;
    foreach(FrequencyBand band in bands)
    {
      //computer our start/end samples.
      band.beginSample = Mathf.FloorToInt(band.beginPercent * samples);
      band.endSample = Mathf.FloorToInt(band.endPercent * samples);
      band.index = index;
      index++;
    }

    foreach(MeterDisplay display in displays)
    {
      Destroy(display.gameObject);
    }

    displays = new MeterDisplay[bands.Count];
    for(int i = 0; i < bands.Count; i++)
    {
      bands[i].peakValue = 0.0f;
      displays[i] = (MeterDisplay)Instantiate(meterDisplayPrefab);
      displays[i].Init(bands[i]);
      displays[i].transform.position = new Vector3(i,0.0f, 0.0f);

    }
  }

  void Update()
  {
    // topFreq = Analyze();
    // midiCode = FrequencyToMIDI(topFreq, 440f, 500);
    check();
    if(isRefresh)
    {
      isRefresh = false;
      refresh();
    }
  }

  public void check()
  {
    aSource.GetSpectrumData(data, 0, FFTWindow.BlackmanHarris);
    //overall loudness
    peakSample = 0.0f;
    foreach(FrequencyBand band in bands)
    {
      band.peakValue = 0.0f;
      for(int i = band.beginSample; i < band.endSample; i++ )
      {
        float d = data[i];
        band.peakValue = d > band.peakValue ? d : band.peakValue;
      }
      peakSample = band.peakValue > peakSample ? band.peakValue : peakSample;
    }
  }
}
