using UnityEngine;
using System;
using System.Collections.Generic;
using System.Collections;

public class FrequencyInfo : MonoBehaviour
{
  public AudioSource aSource;
  public List<float> frequencies;

  public float topFreq;
  public int midiCode;

  public float peakSample = 0.0f;
  // Min = 64. Max = 8192.  
  public int samples = 256;

  public float minSampleVolume = 0.0f;
  
  float[] data;

  public List<FrequencyBand> bands;

  float log = 2;

  public MeterDisplay meterDisplayPrefab;
  MeterDisplay[] displays;

  public FFTWindow window;

  void Start()
  {
    data = new float[samples];
    bands = new List<FrequencyBand>();

    int n = samples;
    int k = 0;
    for(int j=0;j<samples;j++)
    {
      FrequencyBand band = new FrequencyBand();
      band.index = k;
      bands.Add(band);
      band.endSample = n;
      n = Mathf.FloorToInt((float)n / log);
      band.beginSample = n;
      if(n <= 0){
        band.endSample = 0;
         break;
        }
      k++;
    }
    displays = new MeterDisplay[k+1];
    for(int i = 0; i < bands.Count; i++)
    {
      displays[i] = (MeterDisplay)Instantiate(meterDisplayPrefab);
      displays[i].Init(bands[i]);
      displays[i].transform.SetParent(transform, false);
      displays[i].transform.localPosition = new Vector3(i-5.0f,3f, 0.0f);


    }
    

  }

  void Update()
  {
    // topFreq = Analyze();
    // midiCode = FrequencyToMIDI(topFreq, 440f, 500);
    checkBands();
  }


  public void checkBands()
  {
    aSource.GetSpectrumData(data, 0, window);
    //overall loudness
    peakSample = 0.0f;
    foreach(FrequencyBand band in bands)
    {
      band.peakValue = 0.0f;
      for(int i = band.beginSample; i < band.endSample; i++ )
      {
        float d = data[i];
        band.peakValue = d > band.peakValue ? d : band.peakValue;
      }
      peakSample = band.peakValue > peakSample ? band.peakValue : peakSample;
    }
  }

  public void check()
  {
    aSource.GetSpectrumData(data, 0, window);
    int k = 0;
    int crossover = 2;
    peakSample = 0.0f;
    for(int i = 0; i < samples; i++)
    {
      float d = data[i];
      FrequencyBand band = bands[k];
      float b = band.peakValue;
      if(d < minSampleVolume)
      {
        d = 0.0f;
      }

      band.peakValue = (d>b) ? d : b;
      peakSample = d > peakSample ? d : peakSample;
      if(i > crossover - 3)
      {
        k++;
        crossover = Mathf.FloorToInt((float) crossover * log);
        band.peakValue = 0;
      }
    }
  }

  public int FrequencyToMIDI(float freq, float tuningA = 440f, double bandSize=69.5){

    return (int)(bandSize + (12 * (Mathf.Log (freq / tuningA, 2))));

  }


  public float Analyze(){

    float max = 0f;
    int maxIndex = 0;

    aSource.GetSpectrumData (data, 0, FFTWindow.Rectangular);

    for (int i = 0; i < samples; i++) {

      if (data [i] > max) {

        maxIndex = i;
        max = data [i];

      }

    }

    float freq = (float)maxIndex;

    if (maxIndex > 0 && maxIndex < samples) {

      float x = data [maxIndex - 1] / data [maxIndex];
      float y = data [maxIndex + 1] / data [maxIndex];
      freq += 0.5f * (y * y - x * x);

    }

    return freq * (float)AudioSettings.outputSampleRate / samples; // convert index to frequency

  }
}
